import java.io.*;
import java.sql.*;
import java.util.*;

public class DerbyTest extends Thread {
	private int numberOfPersons;

	private String tableName;

	private int waitSeconds = 60;

	private int numberOfThreads = 20;

	private int numberOfRuns = 200;

	private int[] tArray, eArray;

	private Connection conn;

	private FileWriter f;

	public DerbyTest(String tableName, int numberOfPersons, int test,
			boolean scan, double scanEvery, int pageCacheSize) throws Exception {
		this.numberOfPersons = numberOfPersons;
		this.tableName = tableName;
		initializeLists(test);
		initializeFiles(scan, scanEvery, test);

		Properties p = System.getProperties();
		// Setting page size
		p.put("derby.storage.pageSize", "4096");
		p.put("derby.storage.pageCacheSize", "" + pageCacheSize);

		System.out.println("Clock");
		startTest("Clock", test);
		System.out.println("LRU");
		startTest("LRU", test);
		System.out.println("LRU2Q");
		startTest("LRU2Q", test);

		printResults();

	}

	private void startTest(String cacheName, int test) throws Exception {
		FileWriter file = new FileWriter("PageCache.txt");
		file.write(cacheName);
		file.close();

		Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
		conn = DriverManager.getConnection("jdbc:derby:testdb;create=true");
		
		for (int i = 0; i < numberOfThreads; i++) {
			DerbyThread d = new DerbyThread(i);
			d.start();
		}
		sleep(waitSeconds * 1000);

		try {
			conn = DriverManager
					.getConnection("jdbc:derby:testdb;shutdown=true");
		} catch (Exception e) {
		}
	}
	
	private void initializeLists(int test) {
		if (test == 1) {
			ArrayList<Integer> twenty = new ArrayList<Integer>();
			ArrayList<Integer> eighty = new ArrayList<Integer>();

			int j = 0, k = 0;
			for (int i = 0; i < numberOfPersons; i++) {
				double rand = Math.random();
				if (rand < 0.2)
					twenty.add(j++, i);
				else
					eighty.add(k++, i);
			}
			tArray = new int[twenty.size()];
			j = 0;
			for (Iterator iter = twenty.iterator(); iter.hasNext();) {
				int element = (Integer) iter.next();
				tArray[j++] = element;
			}

			eArray = new int[eighty.size()];
			j = 0;
			for (Iterator iter = eighty.iterator(); iter.hasNext();) {
				int element = (Integer) iter.next();
				eArray[j++] = element;
			}
		} else if (test == 2) {
			int i = 0;
			tArray = new int[numberOfPersons / 5];
			for (; i < tArray.length; i++) {
				tArray[i] = i;
			}

			eArray = new int[numberOfPersons * 4 / 5];
			for (int index = 0; index < eArray.length; index++) {
				eArray[index] = i++;
			}
		} else if (test == 4) {
			tArray = new int[numberOfPersons];
			for (int i = 0; i < tArray.length; i++) {
				tArray[i] = i;
			}
		}
	}

	private void initializeFiles(boolean scan, double scanEvery, int test) {
		if (test != 4) {
			for (int j = 0; j < numberOfThreads; j++) {
				try {
					f = new FileWriter("Thread" + j + ".txt");

					for (int i = 0; i < numberOfRuns; i++) {
						i++;
						double rand = Math.random();
						if (scan) {
							if (rand < scanEvery) {
								String sql = "Select * FROM " + tableName
										+ "\n";
								f.write(sql);
							}
						}
						if (rand < 0.8) {
							int indeks = (int) (Math.random() * (tArray.length));
							int verdi = tArray[indeks];
							String sql = "Select * FROM " + tableName
									+ " WHERE id=" + verdi + "\n";
							f.write(sql);
						} else {
							int indeks = (int) (Math.random() * (eArray.length));
							int verdi = eArray[indeks];
							String sql = "Select * FROM " + tableName
									+ " WHERE id=" + verdi + "\n";
							f.write(sql);
						}
					}
					f.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} else { // test==4
			try {

				for (int j = 0; j < numberOfThreads; j++) {
					f = new FileWriter("Thread" + j + ".txt");

					for (int i = 0; i < numberOfRuns; i++) {
						double rand = Math.random();
						if (scan) {
							if (rand < scanEvery) {
								String sql = "Select * FROM " + tableName
										+ "\n";
								f.write(sql);
							}
						}
						int indeks = (int) (Math.random() * (tArray.length));
						int verdi = tArray[indeks];
						String sql = "Select * FROM " + tableName
								+ " WHERE id=" + verdi + "\n";
						f.write(sql);
					}
					f.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
	}

	private void printResults() throws Exception {
		FileReader f = new FileReader("clockStat.txt");
		BufferedReader b = new BufferedReader(f);
		System.out.println("Testresultater:");
		System.out.println("Clock:\t" + b.readLine());
		f = new FileReader("lruStat.txt");
		b = new BufferedReader(f);
		System.out.println("LRU\t" + b.readLine());
		f = new FileReader("lru2QStat.txt");
		b = new BufferedReader(f);
		System.out.println("LRU-2Q\t" + b.readLine());
	}

	public static void main(String[] args) {
		try {
			System.out.println("Creating test tables...");
			new CreateTestTables();
			
			System.out.println("Running test 1...");
			for (int j = 1; j <= 10; j++) {
				System.out.println("CacheSize: " + 40 * j);
				for (int i = 1; i <= 10; i++) {
					new DerbyTest("Table1", 400, 1, true, 0.05, 40 * j);
				}
			}

			System.out.println("Running test 2...");
			for (int j = 10; j <= 10; j++) {
				System.out.println("CacheSize: " + 40 * j);
				for (int i = 1; i <= 2; i++) {
					new DerbyTest("Table2", 4000, 1, true, 0.05, 40 * j);
				}
			}

			System.out.println("Running test 3...");
			System.out.println("0%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, false, 0, 80);
			}

			System.out.println("1%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.01, 80);
			}

			System.out.println("5%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.05, 80);
			}

			System.out.println("10%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.1, 80);
			}

			System.out.println("20%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.2, 80);
			}

			System.out.println("Running test 4...");
			System.out.println("0%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, false, 0, 160);
			}

			System.out.println("1%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.01, 160);
			}

			System.out.println("5%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.05, 160);
			}

			System.out.println("10%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.10, 160);
			}

			System.out.println("20%");
			for (int i = 1; i <= 10; i++) {
				new DerbyTest("Table1", 400, 1, true, 0.2, 160);
			}

			System.out.println("Running test 5...");
			for (int j = 10; j <= 10; j++) {
				System.out.println("CacheSize: " + 40 * j);
				for (int i = 1; i <= 7; i++) {
					new DerbyTest("Table1", 400, 4, true, 0.05, 40 * j);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
